//------------------------------------------------------------------------------
// Contains additional functins:
//
//   - INDEX (read)
//   - NAME (read)
//
// (c) 2000, 2001, Jesco Topp & Andy Ford
//------------------------------------------------------------------------------
unit UTPFF_Aux_Functions;

interface

uses
  Windows,
  UTPFF_Aux_Classes;

//------------------------------------------------------------------------------
// functions to read INDEX and NAME

  //INDEX
  function  Read_INDEX(var F: file): longint;                   //Read
  procedure Write_INDEX(const Index : longint; var F : File);   //Write
  function  SizeOf_INDEX(const Index : longint): integer;       //Get Size of INDEX
  //NAME
  function  Read_NAME(var F: file; PkgVer : integer): string; //Read
  procedure Write_NAME(const Name : string; var F : File);    //Write (only PkgVer = 69)

//------------------------------------------------------------------------------

implementation

//------------------------------------------------------------------------------
// Read INDEX-Format
function Read_INDEX;
var
  B0,B1,B2,B3,B4 : BYTE;
  D0,D1,D2,D3,D4 : longint;
  INDEX          : longint;
  IsNegative     : Boolean;
begin
  B0 := 0;
  B1 := 0;
  B2 := 0;
  B3 := 0;
  B4 := 0;

  BlockRead(F, B0, 1);
  if (B0 > $80) then
  begin
    B0 := B0 - $80;
    IsNegative := TRUE;
  end
  else
    IsNegative := FALSE;

  //Read Following Bytes
  if (B0 >= $40) then
  begin
    B0 := B0 - $40;
    BlockRead(F, B1, 1);
    if (B1 >= $80) then
    begin
      B1 := B1 - $80;
      BlockRead(F, B2, 1);
      if (B2 >= $80) then
      begin
        B2 := B2 - $80;
        BlockRead(F, B3, 1);
        if (B3 >= $80) then
        begin
          B3 := B3 - $80;
          BlockRead(F, B4, 1);
        end;
      end;
    end;
  end;

  //Assign DWORDs
  D0 := B0;
  D1 := B1;
  D2 := B2;
  D3 := B3;
  D4 := B4;
  //Shift Bytes
  D1 := D1 shl 6;
  D2 := D2 shl 13;
  D3 := D3 shl 20;
  D4 := D4 shl 27;

  //Create resulting DWORD
  INDEX := D0 + D1 + D2 + D3 + D4;

  //Check for sign
  if (IsNegative = TRUE) then INDEX := INDEX * (-1);

  Result := INDEX;
end;

procedure Write_INDEX;
var
  B0,B1,B2,B3,B4 : Variant;
  D0,D1,D2,D3,D4 : longint;
  length         : integer;
  IsNegative     : Boolean;
  Value          : longint;
begin
  // check sign and remove it from original DWORD if necessary
  if ( Index < 0 ) then
  begin
    Value := Index * -1;
    IsNegative := TRUE;
  end
  else
  begin
    Value := Index;
    IsNegative := FALSE;
  end;

  // Create BYTEs from DWORD
  B0 := BYTE(Value);
  if ( B0 >= $80 ) then
    B0 := B0 - $80;
  if ( B0 >= $40 ) then
    B0 := B0 - $40;
  Value := Value shr 6;
  B1 := BYTE(Value);
  if ( B1 >= $80 ) then
    B1 := B1 - $80;
  Value := Value shr 7;
  B2 := BYTE(Value);
  if ( B2 >= $80 ) then
    B2 := B2 - $80;
  Value := Value shr 7;
  B3 := BYTE(Value);
  if ( B3 >= $80 ) then
    B3 := B3 - $80;
  Value := Value shr 7;
  B4 := BYTE(Value);
  if ( B4 >= $80 ) then
    B4 := B4- $80;

  // Assign sign-bit
  if ( IsNegative ) then
    B0 := B0 + $80;

  // Assign length-bits
  length := 1;
  if ( B1 + B2 + B3 + B4 > 0 ) then
  begin
    B0 := B0 + $40;
    length := length + 1;
    if ( B2 + B3 + B4 > 0 ) then
    begin
      B1 := B1 + $80;
      length := length + 1;
      if (  B3 + B4 > 0 ) then
      begin
        B2 := B2 + $80;
        length := length + 1;
        if ( B4 > 0 ) then
        begin
          length := length + 1;
          B3 := B3 + $80;
        end;
      end;
    end;
  end;

  D0 := B0;
  // Write INDEX to disk
  BlockWrite(F, D0, 1);
  if ( length > 1 ) then
  begin
    D1 := B1;
    BlockWrite(F, D1, 1);
    if ( length > 2 ) then
    begin
      D2 := B2;
      BlockWrite(F, D2, 1);
      if ( length > 3 ) then
      begin
        D3 := B3;
        BlockWrite(F, D3, 1);
        if ( length > 4 ) then
        begin
          D4 := B4;
          BlockWrite(F, D4, 1);
        end;
      end;
    end;
  end;
end;

function SizeOf_INDEX;
var
  B0,B1,B2,B3,B4 : BYTE;
  length         : integer;
  Value          : DWORD;
begin
  // Create BYTEs
  if ( Index < 0 ) then
  begin
    Value := Index * -1;
  end
  else
  begin
    Value := Index;
  end;

  // Create BYTEs from DWORD
  B0 := BYTE(Value);
  if ( B0 >= $80 ) then
    B0 := B0 - $80;
  if ( B0 >= $40 ) then
    B0 := B0 - $40;
  Value := Value shr 6;
  B1 := BYTE(Value);
  if ( B1 >= $80 ) then
    B1 := B1 - $80;
  Value := Value shr 7;
  B2 := BYTE(Value);
  if ( B2 >= $80 ) then
    B2 := B2 - $80;
  Value := Value shr 7;
  B3 := BYTE(Value);
  if ( B3 >= $80 ) then
    B3 := B3 - $80;
  Value := Value shr 7;
  B4 := BYTE(Value);
  if ( B4 >= $80 ) then
    B4 := B4- $80;

  // length
  length := 1;
  if ( B1 > 0 ) then
  begin
    length := length + 1;
    if ( B2 > 0 ) then
    begin
      length := length + 1;
      if (  B3 > 0 ) then
      begin
        length := length + 1;
        if ( B4 > 0 ) then
        begin
          length := length + 1;
        end;
      end;
    end;
  end;

  Result := length;
end;

//------------------------------------------------------------------------------
// Read NAME-Format
function Read_NAME;
var
  zero : BYTE;
  i    : integer;
  NAME : string;
  temp : char;
begin
  i := 0;
  if (PkgVer < 64) then
  begin
    repeat
    i := i + 1;
    BlockRead(F, zero, 1);
    if (zero <> $00) then
    begin
      Seek(F, FilePos(F) - 1);
      BlockRead(F, temp, 1);
      Insert(temp, NAME, i);
    end;
    until (zero = $00);
  end
  else
  begin
    Seek(F, FilePos(F) + 1); //quit length byte
    repeat
    i := i + 1;
    BlockRead(F, zero, 1);
    if (zero <> $00) then
    begin
      Seek(F, FilePos(F) - 1);
      BlockRead(F, temp, 1);
      Insert(temp, NAME, i);
    end;
    until (zero = $00);
  end;

  Result := NAME;
end;

// supports only PkgVer 69
procedure Write_Name;
var
  tempBYTE : BYTE;
  tempCHAR : char;
  i        : integer;
begin
  tempBYTE := length(Name) + 1;
  BlockWrite(F, tempBYTE, 1);
  for i:=0 to length(Name)-1 do
  begin
    tempCHAR := Name[i+1];
    BlockWrite(F, tempCHAR, 1);
  end;
  tempBYTE := $00;
  BlockWrite(F, tempBYTE, 1);
end;

//------------------------------------------------------------------------------

end.
